/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

#ifndef UKUI_MENU_MENU_EXTENSION_LOADER_H
#define UKUI_MENU_MENU_EXTENSION_LOADER_H

#include <QHash>
#include <QString>
#include <QDir>

#include "widget-extension.h"
#include "context-menu-extension.h"

namespace UkuiMenu {

class MenuExtensionPlugin;

class MenuExtensionLoader
{
public:
    static MenuExtensionLoader *instance();

    QList<WidgetExtension*> widgets() const;
    QList<ContextMenuExtension*> menus() const;

private:
    MenuExtensionLoader();
    void loadInternalExtension();
    void loadExtensionFromDisk();
    void loadExtensionFromDir(QDir dir);

    void setBlackList(const QStringList &blackList);
    void load();
    void expand();

    void registerExtension(MenuExtensionPlugin *plugin);

private:
    QStringList m_blackList;
    QList<WidgetExtension*> m_widgets;
    QList<ContextMenuExtension*> m_menus;
    static QHash<QString, MenuExtensionPlugin*> plugins;
};

} // UkuiMenu

#endif //UKUI_MENU_MENU_EXTENSION_LOADER_H
